// ---------------------------------------------------------------------------
// throw.cs -- Version 2.3 -- amended October 27, 2000
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// New ways to throw stuff -- depending on the binds you set up, you can:
//
// Hold a key and release it to throw a grenade (or a mine) normally.
// A power meter is shown on the screen to help you guage the strength at which
// the grenade (or mine) will be thrown.
//
// Quickly tap a key to throw all the grenades (or mines) you have at full
// force.
//
// Press that key again to stop throwing the grenades (or mines).
//
// Press or tap the key for another item to immediately start throwing that
// item instead.
//
// You can throw one item at full force by tapping the key for that item
// twice -- BAP BAP, off goes one item.
//
// Or set a key that when tapped, throws an item at full force, and when
// pressed normally, performs a normal "metered" throw.
//
// * You can also set a key to throw items while that key is held down,
//   or set it to throw all items until it's pressed again, or set it
//   to display a guage when you press the key, and throw items at the
//   displayed throw strength when you release the key.
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\throw.cs", "2.3", "Lorne Laliberte", "Oct 27, 2000");

include("writer\\event.cs");
include("writer\\item_events.cs");
include("writer\\save.cs");
include("writer\\station_events.cs");
include("writer\\whatif.cs");
include("writer\\you_died.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_throw.cs file

    // Tap key to throw all grenades, press key to throw one grenade
    //bind(keyboard0, make, "f", to, "tapThrow::pressed(Grenade);");
    //bind(keyboard0, break, "f", to, "tapThrow::released(Grenade);");

    // Tap key to throw all mines, press key to throw one mine
    //bind(keyboard0, make, l_shift, "f", to, "tapThrow::pressed(Mine);");
    //bind(keyboard0, break, l_shift, "f", to, "tapThrow::released(Mine);");

    // Hold key to set force using a power guage, release key to throw 1 grenade at that force
    //bind(keyboard0, make, "whatever", TO, "tapThrow::pressed(Grenade, -1);");
    //bind(keyboard0, break, "whatever", TO, "tapThrow::released(Grenade);", tapThrow_guage_one_grenade_break);

    // Hold key to set force using a power guage, release key to throw 1 mine at that force
    //bind(keyboard0, make, "whatever", TO, "tapThrow::pressed(Mine, -1);");
    //bind(keyboard0, break, "whatever", TO, "tapThrow::Released(Mine);", tapThrow_guage_one_mine_break);

    // Hold key to throw all grenades at specified force from 1-100 until key is released
    //bind(keyboard0, make, "whatever", TO, "throwAll::pressed(Grenade, 100);", throwAll_grenades_at_force_make);
    //bind(keyboard0, break, "whatever", TO, "throwAll::released(Grenade);", throwAll_grenades_at_force_break);

    // Hold key to throw all mines at specified force from 1-100 until key is released
    //bind(keyboard0, make, "whatever", TO, "throwAll::pressed(Mine, 100);", throwAll_mines_at_force_make);
    //bind(keyboard0, break, "whatever", TO, "throwAll::released(Mine);", throwAll_mines_at_force_break);

    // Press key to throw all grenades at specified force from 1-100, press again to stop
    //bind(keyboard0, make, "whatever", TO, "throwAll::pressed(Grenade, 100);", throwAll_grenades_at_force_toggle_make);
    //bind(keyboard0, break, "whatever", TO, "", throwAll_grenades_at_force_toggle_break);

    // Press key to throw all mines at specified force from 1-100, press again to stop
    //bind(keyboard0, make, "whatever", TO, "throwAll::pressed(Mine, 100);",throwAll_mines_at_force_toggle_make);
    //bind(keyboard0, break, "whatever", TO, "",throwAll_mines_at_force_toggle_make);

    // Hold key to set force using a power guage, release key to throw all grenades at that force
    //bind(keyboard0, make, "whatever", TO, "throwAll::pressed(Grenade);", throwAll_guage_grenades_make);
    //bind(keyboard0, break, "whatever", TO, "throwAll::released(Grenade);", throwAll_guage_grenades_break);

    // Hold key to set force using a power guage, release key to throw all mines at that force
    //bind(keyboard0, make, "whatever", TO, "throwAll::pressed(Mine);", throwAll_guage_mines_make);
    //bind(keyboard0, break, "whatever", TO, "throwAll::released(Mine);", throwAll_guage_mines_break);

    // Throw 1 grenade at a specific force (1-100)
    //bind(keyboard0, make, l_alt, "f", to, "throwOne(Grenade, 50);", "throwOne_grenade_make");
    //bind(keyboard0, break, l_alt, "f", to, "", "throwOne_grenade_break");

    // Throw 1 mine at a specific force (1-100)
    //bind(keyboard0, make, alt, "whatever", TO, "throwOne(Mine, 100);", throwOne_mine_make);
    //bind(keyboard0, break, alt, "whatever", TO, "", throwOne_mine_break);

    // Tap key to throw one grenade at full force, press key to throw one grenade with metered force
    //bind(keyboard0, make, "whatever", to, "tapThrow::pressed(Grenade);", tapThrowOrMetered_grenade_make);
    //bind(keyboard0, break, "whatever", to, "tapThrow::released(Grenade, 100, true);", tapThrowOrMetered_grenade_break);

    // Tap key to throw one mine at full force, press key to throw one mine with metered force
    //bind(keyboard0, make, "whatever", to, "tapThrow::pressed(Mine);", tapThrowOrMetered_mine_make);
    //bind(keyboard0, break, "whatever", to, "tapThrow::released(Mine, 100, true);", tapThrowOrMetered_mine_break);

// ---------------------------------------------------------------------------

WriterPrefs::load("throw");

function throwOne(%itemname, %strength)
{
    $keepThrowing = false;
    throwRelease(%itemname, %strength);
}


function throwAll::pressed(%itemname, %strength)
{
    $Throw::powerbar = "";
    $throwAll::holding = false;

    // stop throwing
    schedule::cancel(keepThrowing);

    if($keepThrowing && $throwAll::throwing == %itemname)
    {
        $tapThrow::throwing = "";
        $keepThrowing = false;
        return;
    }

    $keepThrowing = false;

    if(!getItemCount(%itemname))
        return;

    if(%strength == "")
    {
        // guage strength, then start throwing
        $keepThrowing = true;
        $throwAll::holding = true;
        throwStart(%itemname);
        return;
    }

    // start throwing
    $throwAll::throwing = %itemname;
    $keepThrowing = true;
    throwRelease(%itemname, %strength);
}


function throwAll::released(%itemname)
{
    if($throwAll::holding)
    {
        $throwAll::throwing = %itemname;

        throwRelease(%itemname);
        return;
    }

    if($keepThrowing)
    {
        // stop previous throwing
        schedule::cancel(keepThrowing);
        $keepThrowing = false;
        $tapThrow::throwing = "";
    $Throw::powerbar = "";
    $throwAll::holding = false;
        return;
    }
}


// Tap key to throw all you're carrying of an item (such as all Grenades or Mines),
// press key and release to throw a single item (such as a single Grenade or Mine).
//
function tapThrow::pressed(%itemname, %taptime)
{
    if($Station::AtStation)
        return;

    if(%taptime == "")
        %taptime = 0.100; // 0.100 seconds default

    // stop previous throwing
    schedule::cancel(keepThrowing);
    $keepThrowing = false;

    if(!getItemCount(%itemname))
        return;

    $Throw::powerbar = "";

    if(%taptime == -1)
    {
        throwStart(%itemname);
        return;
    }

    %simtime = getSimTime();

    tap(throw, %taptime);
    schedule::add("throwStart(\"" @ %itemname @ "\"," @ %simtime @ ");", %taptime + 0.01, tapthrow);
}


function tapThrow::released(%itemname, %strength, %single)
{
    if($Station::AtStation)
        return;

    if(!getItemCount(%itemname))
        return;

    if(wasTapped(throw))
    {
        schedule::cancel(tapthrow);

        if($tapThrow::throwing == %itemname)
        {
            $tapThrow::throwing = "";
            return;
        }

        if(%single)
           $keepThrowing = false;
        else
            $keepThrowing = true;

        $tapThrow::throwing = %itemname;

        if(%strength == "")
            %strength = 100;

        throwRelease(%itemname, %strength);
    }
    else // key was held then released
    {
        // Calculate how many of this item we'll have left...the item count
        // is not updated right after throwRelease(), so let's sidestep the delay
        %count = getItemCount(%itemname) - 1;

        $tapThrow::throwing = "";

        // Throw it
        throwRelease(%itemname);

        // Show how many of this item are left
        remoteBP(2048, "<L5><f2>" @ wif(%count > 0, %count, "NO") @ "<f0> " @ %itemname @ wif(%count == 1, " left", "s left"), 3);
    }
}


function tapThrow::onThrowReleased(%itemtype, %itemname, %strength)
{
    if($keepThrowing)
    {
        // Get number of items before and after throwing one
        %countafter = (%count = getItemCount(%itemname)) - 1;

        // Show how many of this item are remaining
    	remoteBP(2048, "<L5><f2>" @ wif(%countafter > 0, %countafter, "NO") @ "<f0> " @ %itemname @ wif(%countafter == 1, " left", "s left"), 3);

        if(%count) // we have some left to throw
            schedule::add("throwRelease(\"" @ %itemname @ "\"," @ %strength @ ");", 0.6, keepThrowing);
        else
            $tapThrow::throwing = "";
    }
    else
    {
        schedule::cancel(keepThrowing);
        $tapThrow::throwing = "";
    }
    return;
}


function tapThrow::onThrowPower(%itemtype, %itemname, %strength)
{
    if(!$throwpower)
        return;

    $Throw::powerbar = $Throw::powerbar @ "||";

    if(%strength > 7)
        remoteBP(2048, "<L5>" @ %itemname @ "  <f2>" @ %strength @ "% <f1><L25>" @ $Throw::powerbar, 4);

    return;
}


function throw::onYouDied()
{
    schedule::cancel(keepThrowing);
    $keepThrowing=false;
    $tapThrow::throwing="";
    return;
}

function throw::save()
{
    AddExportText("    // Tap key to throw all grenades, press key to throw one grenade");

    ExportBind("tapThrow::pressed(Grenade);", "//bind(keyboard0, make, \"f\", TO, \"tapThrow::pressed(Grenade);\");");
    ExportBind("tapThrow::released(Grenade);", "//bind(keyboard0, break, \"f\", TO, \"tapThrow::released(Grenade);\");");

    AddExportText("");
    AddExportText("    // Tap key to throw all mines, press key to throw one mine");

    ExportBind("tapThrow::pressed(Mine);", "//bind(keyboard0, make, l_shift, \"f\", TO, \"tapThrow::pressed(Mine);\");");
    ExportBind("tapThrow::released(Mine);", "//bind(keyboard0, break, l_shift, \"f\", TO, \"tapThrow::released(Mine);\");");

    AddExportText("");
    AddExportText("    // Hold key to set force using a power guage, release key to throw 1 grenade at that force");

    ExportBind("tapThrow::pressed(Grenade, -1);", "//bind(keyboard0, make, \"whatever\", TO, \"tapThrow::pressed(Grenade, -1);\");");
    ExportBind("tapThrow_guage_one_grenade_break", "//bind(keyboard0, break, \"whatever\", TO, \"tapThrow::released(Grenade);\", tapThrow_guage_one_grenade_break);");

    AddExportText("");
    AddExportText("    // Hold key to set force using a power guage, release key to throw 1 mine at that force");

    ExportBind("tapThrow::pressed(Mine, -1);", "//bind(keyboard0, make, \"whatever\", TO, \"tapThrow::pressed(Mine, -1);\");");
    ExportBind("tapThrow_guage_one_mine_break", "//bind(keyboard0, break, \"whatever\", TO, \"tapThrow::Released(Mine);\", tapThrow_guage_one_mine_break);");

    AddExportText("");
    AddExportText("    // Hold key to throw all grenades at specified force from 1-100 until key is released");

    ExportBind("throwAll_grenades_at_force_make", "//bind(keyboard0, make, \"whatever\", TO, \"throwAll::pressed(Grenade, 100);\", throwAll_grenades_at_force_make);");
    ExportBind("throwAll_grenades_at_force_break", "//bind(keyboard0, break, \"whatever\", TO, \"throwAll::released(Grenade);\", throwAll_grenades_at_force_break);");

    AddExportText("");
    AddExportText("    // Hold key to throw all mines at specified force from 1-100 until key is released");

    ExportBind("throwAll_mines_at_force_make", "//bind(keyboard0, make, \"whatever\", TO, \"throwAll::pressed(Mine, 100);\", throwAll_mines_at_force_make);");
    ExportBind("throwAll_mines_at_force_break", "//bind(keyboard0, break, \"whatever\", TO, \"throwAll::released(Mine);\", throwAll_mines_at_force_break);");

    AddExportText("");
    AddExportText("    // Press key to throw all grenades at specified force from 1-100, press again to stop");

    ExportBind("throwAll_grenades_at_force_toggle_make", "//bind(keyboard0, make, \"whatever\", TO, \"throwAll::pressed(Grenade, 100);\", throwAll_grenades_at_force_toggle_make);");
    ExportBind("throwAll_grenades_at_force_toggle_break", "//bind(keyboard0, break, \"whatever\", TO, \"\", throwAll_grenades_at_force_toggle_break);");

    AddExportText("");
    AddExportText("    // Press key to throw all mines at specified force from 1-100, press again to stop");

    ExportBind("throwAll_mines_at_force_toggle_make", "//bind(keyboard0, make, \"whatever\", TO, \"throwAll::pressed(Mine, 100);\",throwAll_mines_at_force_toggle_make);");
    ExportBind("throwAll_mines_at_force_toggle_break", "//bind(keyboard0, break, \"whatever\", TO, \"\",throwAll_mines_at_force_toggle_make);");

    AddExportText("");
    AddExportText("    // Hold key to set force using a power guage, release key to throw all grenades at that force");

    ExportBind("throwAll_guage_grenades_make", "//bind(keyboard0, make, \"whatever\", TO, \"throwAll::pressed(Grenade);\", throwAll_guage_grenades_make);");
    ExportBind("throwAll_guage_grenades_break", "//bind(keyboard0, break, \"whatever\", TO, \"throwAll::released(Grenade);\", throwAll_guage_grenades_break);");

    AddExportText("");
    AddExportText("    // Hold key to set force using a power guage, release key to throw all mines at that force");

    ExportBind("throwAll_guage_mines_make", "//bind(keyboard0, make, \"whatever\", TO, \"throwAll::pressed(Mine);\", throwAll_guage_mines_make);");
    ExportBind("throwAll_guage_mines_break", "//bind(keyboard0, break, \"whatever\", TO, \"throwAll::released(Mine);\", throwAll_guage_mines_break);");

    AddExportText("");
    AddExportText("    // Throw 1 grenade at a specific force (1-100)");

    ExportBind("throwOne_grenade_make", "//bind(keyboard0, make, alt, \"whatever\", TO, \"throwOne(Grenade, 100);\", throwOne_grenade_make);");
    ExportBind("throwOne_grenade_break", "//bind(keyboard0, break, alt, \"whatever\", TO, \"\",throwOne_grenade_break);");

    AddExportText("");
    AddExportText("    // Throw 1 mine at a specific force (1-100)");

    ExportBind("throwOne_mine_make", "//bind(keyboard0, make, alt, \"whatever\", TO, \"throwOne(Mine, 100);\", throwOne_mine_make);");
    ExportBind("throwOne_mine_break", "//bind(keyboard0, break, alt, \"whatever\", TO, \"\", throwOne_mine_break);");

    AddExportText("");
    AddExportText("    // Tap key to throw one grenade at full force, press key to throw one grenade with metered force");

    ExportBind("tapThrowOrMetered_grenade_make", "//bind(keyboard0, make, alt, \"whatever\", TO, \"tapThrow::pressed(Grenade);\", tapThrowOrMetered_grenade_make);");
    ExportBind("tapThrowOrMetered_grenade_break", "//bind(keyboard0, break, alt, \"whatever\", TO, \"tapThrow::released(Grenade, 100, true);\", tapThrowOrMetered_grenade_break);");

    AddExportText("");
    AddExportText("    // Tap key to throw one mine at full force, press key to throw one mine with metered force");

    ExportBind("tapThrowOrMetered_mine_make", "//bind(keyboard0, make, alt, \"whatever\", TO, \"tapThrow::pressed(Mine);\", tapThrowOrMetered_mine_make);");
    ExportBind("tapThrowOrMetered_mine_break", "//bind(keyboard0, break, alt, \"whatever\", TO, \"tapThrow::released(Mine, 100, true);\", tapThrowOrMetered_mine_break);");
}

WriterPrefs::save("throw");

function throw::init()
{
    needs("writer\\throw.cs", "writer\\event.cs 2.7",
                              "writer\\item_events.cs 2.12",
                              "writer\\save.cs 2.3",
                              "writer\\station_events.cs 4.8",
                              "writer\\whatif.cs 1.1",
                              "writer\\you_died.cs 1.6");

    Event::Attach(eventThrowPower, tapThrow::onThrowPower);
    Event::Attach(eventThrowReleased, tapThrow::onThrowReleased);
    Event::Attach(eventYouDied, "throw::onYouDied();");
}
Event::Attach(writerInit, "throw::init();");

if(!$UsingWriterLoader) throw::init();
